@@ -437,10 +437,13 @@ class Any2Id():
         self.fix = fix
         self.use_line_no = use_line_no
         self.counter = Counter() if not counter else counter
-        self.any2id = {}
+        self.any2id = {}  # 内部核心dict
         if exist_dict is not None:
             self.any2id.update(exist_dict)
 
+        # for method_name in dict.__dict__:  # 除了下面显式定义外保证能以dict的各种方法操作Any2id
+        #     setattr(self, method_name, getattr(self.any2id, method_name))
+
     def keys(self):
         return self.any2id.keys()
 
@@ -450,6 +453,9 @@ class Any2Id():
     def items(self):
         return self.any2id.items()
 
+    def pop(self, key):
+        return self.any2id.pop(key)
+
     def __getitem__(self, item):
         return self.any2id.__getitem__(item)
 
@@ -524,16 +530,22 @@ class Any2Id():
         if os.path.exists(file):
             inst.load(file, deli=deli)
         else:
+            # will return inst with empty any2id , e.g. boolean(inst) or len(inst) will return False
             print(f'vocab file: {file} not found, need to build and save later')
         return inst
 
 
 def pad_sequences(sequences, maxlen=None, dtype='int32', padding='pre', truncating='pre', value=0.):
-    assert hasattr(sequences, '__len__')
-    len_lst = []
-    for x in sequences:
-        assert hasattr(x, '__len__')
-        len_lst.append(len(x))
+    if not hasattr(sequences, '__len__') or not all([hasattr(x, '__len__') for x in sequences]):
+        raise ValueError(f'sequences invalid: {sequences}')
+
+    len_lst = [len(x) for x in sequences]
+
+    if len(set(len_lst)) == 1:  # 长度均相等
+        ret = np.array(sequences)
+        if maxlen is not None:
+            ret = ret[:,-maxlen:] if truncating=='pre' else ret[:,:maxlen]
+        return ret
 
     num_samples = len(sequences)
     if maxlen is None:
@@ -578,7 +590,7 @@ def pad_sequences(sequences, maxlen=None, dtype='int32', padding='pre', truncati
     return x
 
 
-def get_file_logger(logger_name, log_file='./zyn.log', level='DEBUG'):
+def get_file_logger(logger_name, log_file='./qiznlp.log', level='DEBUG'):
     level = {
         'ERROR': logging.ERROR,
         'INFO': logging.INFO,
@@ -599,6 +611,10 @@ def get_file_logger(logger_name, log_file='./zyn.log', level='DEBUG'):
 
     return logger
 
+def suppress_tf_warning(tf):
+    import warnings
+    warnings.filterwarnings('ignore')
+    tf.logging.set_verbosity(tf.logging.ERROR)
 
 if __name__ == '__main__':
     check_overlap([1, 2, 3, 4, 5, 6], [4, 5, 6, 7, 8, 9, 0])
